/*
	Vibrator Control Firmware
	Copyright (C) 2003  Matthias Bruestle <m@mbsks.franken.de>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#if (__GNUC__==3) && (__GNUC_MINOR__==0)
#include <eeprom.h>
#include <ina90.h>
#include <io.h>
#else
#include <avr/eeprom.h>
#include <avr/ina90.h>
#include <avr/io.h>
#endif

#include <ser-avr.h>

struct {
	iu8	in_low;
	iu8	in_high;
	iu8	out_low;
	iu8 out_high;
	iu8 delta_in;
	iu8 delta_out;
} pwmconf;

iu8 vibr;

void hal_init( void )
{
	iu8 b;

	/* Enable PWM output */
	sbi( PWM_DDR, PWM_PIN );

	/* LED off */
	sbi( LED_DDR, LED_PIN );
	cbi( LED_PORT, LED_PIN );

	/* 9600bps */
	outp( 0, UBRRH );
	outp( UBR_9600, UBRRL );

	/* Enable UART */
	outp( 0x18, UCSRB );

	vibr=0;
	/* PWM compare value */
	outp( 0x00, OCR1H );
	outp( 0x00, OCR1L );
	/* 8bit non-inverted PWM */
	outp( 0x81, TCCR1A );
	/* Start timer */
	outp( 0x01, TCCR1B );

	hal_eeprom_read( &b, ADDR_INITED, 1 );
	if( b!=0x69 ) {
		/* Not configured yet */
		pwmconf.in_low=1;
		pwmconf.in_high=255;
		pwmconf.out_low=8;
		pwmconf.out_high=100;
		pwmconf.delta_in=254;
		pwmconf.delta_out=92;

		hal_eeprom_write( ADDR_PWMCONF, (iu8*)&pwmconf, 4 );
		b=0x69;
		hal_eeprom_write( ADDR_INITED, &b, 1 );
	} else {
		/* Read configuration */
		hal_eeprom_read( (iu8*)&pwmconf, ADDR_PWMCONF, 4 );
		pwmconf.delta_in=pwmconf.in_high-pwmconf.in_low;
		pwmconf.delta_out=pwmconf.out_high-pwmconf.out_low;
	}
}

bool hal_eeprom_read( iu8 *dst, iu16 src, iu8 len )
{
	while( len-- ) {
		while( !eeprom_is_ready() ) _NOP();

		*dst++ = eeprom_rb( src++ );
	}

	return TRUE;
}

bool hal_eeprom_write( iu16 dst, iu8 *src, iu8 len )
{
	while( len-- ) {
		while( !eeprom_is_ready() ) _NOP();

		eeprom_wb( dst++, *src++ );
	}

	return TRUE;
}

void hal_sendByte( iu8 b )
{
	while( bit_is_clear( UCSRA, UDRE ) ) {}
	outp( b, UDR );
}

iu8 hal_recByte( void )
{
	iu8 b;

	while( bit_is_clear( UCSRA, RXC ) ) {}
	b = inp( UDR );

	return b;
}

void hal_setPwm( iu8 width )
{
	iu8 b, new;
	iu16 s;

	if( width<pwmconf.in_low ) {
		new = 0;
	} else if( width>=pwmconf.in_high ) {
		new = pwmconf.out_high;
	} else {
		b = width-pwmconf.in_low;
		s = (iu16)b * pwmconf.delta_out;
		s /= pwmconf.delta_in;
		new = s + pwmconf.out_low;
	}

	hal_led( new );

	outp( 0x00, OCR1H );

	b = inp( OCR1L );

	if( b < new ) {
		/* Change PWM new */
		outp( new, OCR1L );
	} else {
		while( b>new ) {
			b-=1;
			if( b<pwmconf.out_low ) b=0;
			outp( b, OCR1L );
			for( s=0; s<0x0800; s++ ) _NOP();
		}
	}
}

void hal_led( bool on )
{
	if( on ) {
		sbi( LED_DDR, LED_PIN );
		sbi( LED_PORT, LED_PIN );
	} else {
		sbi( LED_DDR, LED_PIN );
		cbi( LED_PORT, LED_PIN );
	}
}

void hal_die( void )
{
	iu16 i;

	for(;;){
		hal_led( TRUE );
		for( i=0; i<10000; i++ ) _NOP();
		hal_led( FALSE );
		for( i=0; i<10000; i++ ) _NOP();
	}
}

int main( void )
{
	iu8 b;

	hal_init();

	/* ATR */
	hal_sendByte( 'B' );
	hal_sendByte( 'H' );
	hal_sendByte( 'E' );
	hal_sendByte( 'F' );
	hal_sendByte( '1' );

	for(;;) {
		b = hal_recByte( );

		switch( b ) {
		case CMD_SET:
			hal_sendByte( CMD_SET );
			b = hal_recByte( );
			hal_setPwm( b );
			hal_sendByte( b );
			break;
		case CMD_PWMCONF:
			hal_sendByte( CMD_PWMCONF );
			pwmconf.in_low = hal_recByte();
			pwmconf.in_high = hal_recByte();
			pwmconf.out_low = hal_recByte();
			pwmconf.out_high = hal_recByte();
			pwmconf.delta_in=pwmconf.in_high-pwmconf.in_low;
			pwmconf.delta_out=pwmconf.out_high-pwmconf.out_low;
			b = pwmconf.in_low+ pwmconf.in_high+ pwmconf.out_low+
				pwmconf.out_high;
			hal_eeprom_write( ADDR_PWMCONF, (iu8*)&pwmconf, 4 );
			hal_sendByte( b );
		default:
			hal_sendByte( RSP_NAK );
			break;
		}
	}
}

